import { Controller } from '@hotwired/stimulus';

export default class extends Controller {
  static targets = ['submitButton', 'submitText', 'addAction', 'nextAction'];

  connect() {
    this.requiredFields = this.element.querySelectorAll('*[required]');

    if (this.hasAddActionTarget) {
      this.updateSubmitContentHandler = this.updateSubmitContent.bind(this);
      this.addActionTarget.addEventListener('change', this.updateSubmitContentHandler);
      this.nextActionTarget.addEventListener('valueChanged', this.updateSubmitContentHandler);
    }

    this.updateSubmitVisibilityHandler = this.updateSubmitVisibility.bind(this);
    this.requiredFields.forEach(el => {
      ['valueChanged', 'keyup', 'change'].forEach(eventType => {
        el.addEventListener(eventType, this.updateSubmitVisibilityHandler);
      });
    });
  }

  disconnect() {
    if (this.hasAddActionTarget) {
      this.addActionTarget.removeEventListener('change', this.updateSubmitContentHandler);
      this.nextActionTarget.removeEventListener('valueChanged', this.updateSubmitContentHandler);
    }

    this.requiredFields.forEach(el => {
      ['valueChanged', 'keyup', 'change'].forEach(eventType => {
        el.removeEventListener(eventType, this.updateSubmitVisibilityHandler);
      });
    });
  }

  updateSubmitVisibility() {
    const shouldShowSubmit = this.calculateShouldShowSubmit();

    this.submitButtonTarget.classList.toggle('hidden', !shouldShowSubmit);
    this.updateSubmitContent();
  }

  calculateShouldShowSubmit() {
    const allFieldsFilled = Array.from(this.requiredFields).every(el => this.isNotEmptyField(el));
    const isAddActionValid = this.hasAddActionTarget && this.addActionTarget.checked ? !!this.nextActionTarget.value : true;
    return allFieldsFilled && isAddActionValid;
  }

  isNotEmptyField(el) {
    switch (el.type) {
      case 'checkbox':
      case 'radio':
        return el.checked;
      case 'file':
        return el.files.length > 0;
      default:
        return el.value.trim().length > 0;
    }
  }

  updateSubmitContent() {
    const text = this.hasAddActionTarget && this.addActionTarget.checked ? "Valider et passer à l'action suivante" : 'Valider';

    this.submitTextTarget.textContent = text;
  }
}
