import SlimSelect from 'slimselect';

export default function attachSlimSelectTo(element, opts = {}) {
  const normalizeText = (text) => {
    return text
      .toLowerCase()
      .normalize('NFD')
      .replace(/[\u0300-\u036f]/g, '');
  }

  const form = element.closest('form');

  const slimSelectInstance = new SlimSelect({
    select: element,
    settings: {
      placeholderText: '',
      // searchText: 'Aucun résultat',
      searchPlaceholder: 'Rechercher',
    },
    events: {
      afterChange: info => {
        const event = new CustomEvent('valueChanged', { detail: { selectedValues: info } });
        element.dispatchEvent(event);
      },
      searchFilter: (option, search) => {
        const normalizedOption = normalizeText(option.text);
        const normalizedSearch = normalizeText(search);

        return normalizedOption.includes(normalizedSearch);
      },

      ...(opts.events || {}),
    },
  });

  form.addEventListener('reset', () => {
    slimSelectInstance.setSelected([]);
  });

  const detachSlimSelect = (_this) => {
    element.slim.destroy();
    form.removeEventListener('reset', _this);
  }

  return detachSlimSelect
}

