# frozen_string_literal: true

Log.class_eval do
  CREATE_CODES = [
    'DOC_CREATED',
    'DOC_CREATED_FROM_TEMPLATE',
    'DOC_ADDED_TO_COMPLETNESS',
    'OPERATION_CREATED',
    'DIRECTORY_CREATED'
  ].freeze

  DELETE_CODES = [
    'DOC_DELETED'
  ].freeze

  SHARE_CODES = [
    'DOC_SHARED',
    'MULTIPLE_DOC_SHARED',
    'SHARE_LINK_CREATED',
    'COMMENT_ADDED_TO_LINK'
  ].freeze

  CLASSIFY_CODES = [
    'DOC_CLASSIFIED'
  ].freeze

  EDIT_CODES = [
    'DOC_EDITED_IN_OFFICE_ONLINE',
    'DIRECTORY_INFORMATION_UPDATED'
  ].freeze

  PROCESS_CODES = [
    'DOC_VALIDATED',
    'DOC_REFUSED',
    'DOC_ASSIGNED',
    'DOC_SIGNED',
    'DOC_TO_SIGN',
    'DOC_ANSWERED'
  ].freeze

  class << self
    %i[create delete share classify edit process].each do |log_type|
      define_method("#{log_type}_action_log") do |code, opts = {}|
        raise "#{log_type} action code missing" if code.nil?
        raise "#{log_type} action code invalid" unless "#{log_type.upcase}_CODES".constantize.include?(code)

        action_log(code, opts)
      end
    end

    def action_log(code, opts)
      tagset = Tagset.find_by(id: opts[:tagset_id])
      user = User.find_by(id: opts[:user_id])

      message_options = {
        title: opts[:title].presence || tagset&.title,
        username: user&.full_name || user&.email,
        recipients: opts[:recipients].to_a&.compact&.to_sentence(last_word_connector: ' et '),
        link: opts[:link],
        recipient: opts[:recipient],
        directory: opts[:directory]
      }

      action_msg = [I18n.t(code, scope: 'log.action', **message_options)]
      action_msg << opts[:action_msg].presence
      Log.perform_later(
        code:,
        data: 'hubdoc',
        tagset_id: tagset&.id || opts[:tagset_id],
        user_id: user&.id || opts[:user_id],
        action: action_msg.compact.join(' : ')
      )
    end
  end
end
